#ifndef DATA_ANALYSIS_H
#define DATA_ANALYSIS_H

#include "Field_Data_Manager.h"
#include "Forecast_Util.h"

//include GMathLib header
#include <gmathcore/GMatrix.h>
#include <gmathcore/GVector.h>
#include <gfd/GConstants.h>

using namespace GMathLib;
using namespace GMathLib::GFD::GConstants;

/**
 * \񂳂ꂽ(|eVQx, ֐), xExEn̊eGlM[͂NX.
 */
class Data_Analysis
{
public:
    // RXgN^
    Data_Analysis()
    : l(8.64e05)
    {
    }

    /**
     * ςꂽ 500 hPa ̉x̎擾
     */
    double Zonal_Mean_Temparature(Param_List& pm, Field_Data_Manager& dm, int j){
       GVector* p_sf1_mean = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER);
       GVector* p_sf3_mean = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER);

       return pm.f0 * ( p_sf1_mean->Get(j) - p_sf3_mean->Get(j) ) / Earth::R_dry; 
    }

    /**
     * 1000 hPa ̉x̎擾
     */
    double Temparature(Param_List& pm, Field_Data_Manager& dm, int i, int j){
       GMatrix* p_sf1 = dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER1_AFTER);
       GMatrix* p_sf3 = dm.Get_Stream_Func(Field_Data_Manager::SF_LAYER3_AFTER);

       return pm.f0 * ( p_sf1->GetElement(i, j) - p_sf3->GetElement(i, j) ) / Earth::R_dry; 
    }

    /**
     * ςꂽ 250 hPa ̑x̎擾
     */
    double Zonal_Mean_u1(Param_List& pm, Field_Data_Manager& dm, int j){
	GVector* p_sf1_mean = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER);
	
	return zonal_Mean_SF_To_u(pm, p_sf1_mean, j);
    }

    /**
     * ςꂽ 750 hPa ̑x̎擾
     */
    double Zonal_Mean_u3(Param_List& pm, Field_Data_Manager& dm, int j){
	GVector* p_sf3_mean = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER);
	
	return zonal_Mean_SF_To_u(pm, p_sf3_mean, j);
    }

    /**
     * ςꂽ 1000 hPa ̑x̎擾
     * ( 250 hPa, 750 hPa ̑x̓Ϗ`Ԃ )
     */
    double Zonal_Mean_u4(Param_List& pm, Field_Data_Manager& dm, int j){
	GVector* p_sf1_mean = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER1_AFTER);
	GVector* p_sf3_mean = dm.Get_Stream_Func_Mean(Field_Data_Manager::SF_LAYER3_AFTER);
	double u4_mean;

        // 1000 hPa ̃f[^, 250 hPa, 750 hPa ̃f[^ɂ`Ԃɂċ߂.
	u4_mean =  ( 
	            3.0 * zonal_Mean_SF_To_u(pm, p_sf3_mean, j) - zonal_Mean_SF_To_u(pm, p_sf1_mean, j) 
	           ) / 2.0;

	return u4_mean;
    }

    /**
     * ςkx̎擾
     * (  p xꂩ狁߂ )
     */
    double Zonal_Mean_V1(Param_List& pm, Field_Data_Manager& dm, int j){
        GVector* p_omega_m = dm.Get_Omega_Mean();
        double V_jdwn = 0.0;

        for(int j_=1; j_ <= j; j_++){
	    V_jdwn = V_jdwn - pm.del_y * p_omega_m->Get(j_) / pm.p2;
	}
        
	return V_jdwn;
    }
    

    // ̃fɂGlM[̃U[o[, ێĂGlM[ʂ擾邽߂
    // ֐Q
   
    /**
     * ςꂽx̉^GlM[̎擾
     */
    double K_Mean(Param_List& pm, Field_Data_Manager& dm);

    /**
     * x̏̉^GlM[̎擾
     */
    double K_Disturb(Param_List& pm, Field_Data_Manager& dm);
    
    /**
     * ωꂽ|eVGlM[̎擾
     */
    double P_Mean(Param_List& pm, Field_Data_Manager& dm);
    
    /**
     * ̃|eVGlM[ ( APE ) ̎擾
     */
    double P_Disturb(Param_List& pm, Field_Data_Manager& dm);
    


    //  4 ނ̃GlM[̃U[o[o肷GlM[ʂ߂֐Q

    /**
     * fMM(ɂ͈l)瓌ς|eVGlM[ւ̕ϊʂ̎擾
     */
    double Mean_Q_To_Mean_P(Param_List& pm, Field_Data_Manager& dm);
    
    /**
     * ς|eVGlM[瓌ςx̉^GlM[ւ̕ϊʂ̎擾
     */
    double Mean_P_To_Mean_K(Param_List& pm, Field_Data_Manager& dm);
    
    /**
     * ςx̉^GlM[̉QSɂUʂ̎擾
     */
    double Mean_K_To_A(Param_List& pm, Field_Data_Manager& dm);
    
    /**
     * ς|eVGlM[̉QSɂUʂ̎擾
     */
    double Mean_P_To_A(Param_List& pm, Field_Data_Manager& dm);
    
    /**
     * ςx̉^GlM[̎擾
     */
    double Mean_K_To_k(Param_List& pm, Field_Data_Manager& dm);

    /**
     * ς|eVGlM[̃|eVGlM[ւ̕ϊʂ̎擾
     */
    double Mean_P_To_Disturb_P(Param_List& pm, Field_Data_Manager& dm);
    
    /**
     * ̃|eVGlM[̉^GlM[ւ̕ϊʂ̎擾
     */
    double Disturb_P_To_Disturb_K(Param_List& pm, Field_Data_Manager& dm);
    
    /**
     * ̉^GlM[瓌ςx̉^GlM[ւ̕ϊʂ̎擾
     */
    double Disturb_K_To_Mean_K(Param_List& pm, Field_Data_Manager& dm);
    
    /**
     * ̑x̉^GlM[̉QSɂUʂ̎擾
     */
    double Disturb_K_To_A(Param_List& pm, Field_Data_Manager& dm);
    
    /** 
     * ̃|eVGlM[̉QSɂUʂ̎擾
     */
    double Disturb_P_To_A(Param_List& pm, Field_Data_Manager& dm);
    
    /**
     * ̉^GlM[̒nʂ̖Cɂωʂ̎擾
     */
    double Disturb_K_To_k(Param_List& pm, Field_Data_Manager& dm);


private:

    double zonal_Mean_SF_To_u(Param_List& pm, GVector* p_sf_mean, int j){
        double tmp;
	tmp = - ( p_sf_mean->Get(j+1) - p_sf_mean->Get(j-1) ) 
                / ( 2.0 * pm.del_y );

	return tmp;
    }

    double time_ave_value(GMatrix* p_after_mx, GMatrix* p_before_mx, int i, int j){
        
        return ( p_after_mx->Get(i, j) + p_before_mx->Get(i, j) ) / 2.0;
    }

    double time_ave_value(GVector* p_after_mx, GVector* p_before_mx, int j){
        
        return ( p_after_mx->Get(j) + p_before_mx->Get(j) ) / 2.0;
    }

    double l; /* < 10  day. ֐̕ԂlƂȂGlM[̕ϊʂ, 1 ̊Ԃ
                   ϊGlM[ϊʂƂȂ悤, |킷Wł. **/

    // 250 hPa, 750 hPa ̑S̈ɂĕ̑傫 1 m/s ł悤ȂƂ,
    // n^GlM[ 1 PʂƂ.
    // i.e
    // 1 unit = 6.0e13 [J m^2 ]
    const static double ENERGY_UNIT = 6.0e13;
};

#endif
